<html>
  <head>
    <title>wcrelated.py</title>
  </head>
  <body>
    <h1>get related</h1>
    <form method="post" action="/related">
    <p><input type="text" name="oclcnum"/><input type="submit" value="submit"/></p>
    </form>
  </body>
 </html>